;*****************************************************************************
;				NICST TOOL
;
;  RC-5 Part 2 test for TEB bands. Calculates the background subtracted DN and
;  standard deviations. Only calculates on a scan by scan basis.
;
;  Input:
;  1) File with inpath and outpath.
;
;  Output:
;  1) slope1(collect,band,HAM,detector,subsample,scan,2] -- coefficients from a straight line fit to
;     the dn versus frame.
;  2) slope1_obc(collect,band,HAM,detector,subsample,scan,2] -- coefficients from a straight line fit to
;     the dn_obc versus frame.
;  3) dn_col(collect,band,HAM,detector,subsample,scan,3] -- dn for frame average with a 3 sigma kick 
;     out at each step for (0=EV, 1=SV, 2=OBC). The EV and OBC indices are SV subtracted.
;  4) std_col(collect,band,HAM,detector,subsample,scan,3] -- std for frame average with a 3 sigma kick 
;     out at each step for (0=EV, 1=SV, 2=OBC). The EV and OBC indices are SV subtracted.
;  
;  v1.0 - 07/02/2008 - Jeff McIntire
;*****************************************************************************

PRO RC5_OBC_dn_3, infile

;*****************************************************************************
;  Read in input file, sav TEB file, and get list of dump files.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.
fnumber=strarr(1)
header_size=intarr(1)

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,fnumber
readf,lun,header_size
Free_Lun,lun

n_collect,inpath,ncollects,type,1

restore,'/home/jmcintire/VIIRS_Data/general.TEB.sav'

; Get list of .dmp files from specified path. 
spawn, 'ls -C1 ' + inpath + '*.dmp', list_files
IF N_ELEMENTS(list_files) EQ 0 THEN goto,skipout
nfile=N_ELEMENTS(list_files)

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

max_scan=100					; Total # scans.

outfile=' '					; Output file name, which is reasigned.

; DN values per collect (0=EV, 1=SV, 2=OBC).
dn_col=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan,3)

; STD values per collect (0=EV, 1=SV, 2=OBC).
std_col=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan,3)

; Drifting check.
slope1=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan,2)
slope_obc1=dblarr(ncollects,nbands_TEB,nham,maxdet,maxsubsets,max_scan,2)

;*****************************************************************************
;  Read in the dump files for SV.
;*****************************************************************************

m=0
a5=0
FOR ifile=0, nfile-1 DO BEGIN

	nlines=File_Lines(list_files[ifile])
	IF nlines EQ 0 THEN continue

	a2=strsplit(list_files[ifile],'/',/Extract)
	a3=N_ELEMENTS(a2)
	a4=strmid(a2[a3-1],strpos(a2[a3-1],'_',/Reverse_Search)+1,1)		; Determine I or M band.
	a8=strmid(a2[a3-1],strpos(a2[a3-1],'/',/Reverse_Search)+10,2)		; Determine EV or SV.

	; Keep only the SV.
	IF a8 NE 'SV' THEN continue

	; Determine the relative collect #.
	m=m+1
	IF m EQ 9 THEN BEGIN
		a5=a5+1
		m=1
	ENDIF
	print,list_files[ifile]

	read_dmpd2,list_files[ifile],header_size,uaid,collect,scan,ham,apid,band,dnb_s,det,dsect,frame,offset,gain_type,dns

	; Translate the band name to a number.
	IF band[0] EQ 'I4' THEN b=0
	IF band[0] EQ 'I5' THEN b=1
	IF band[0] EQ 'M12' THEN b=2
	IF band[0] EQ 'M13' THEN b=3
	IF band[0] EQ 'M14' THEN b=4
	IF band[0] EQ 'M15' THEN b=5
	IF band[0] EQ 'M16A' THEN b=6
	IF band[0] EQ 'M16B' THEN b=7

	IF band_type_TEB[b] EQ 1 THEN max_frame=96		; Total # frames (I band).
	IF band_type_TEB[b] EQ 0 THEN max_frame=48		; Total # frames (M band).

;*****************************************************************************
;  Write DN values to an array for SV.
;*****************************************************************************

	dn_frame=dblarr(nham,maxdet,maxsubsets,max_scan,max_frame)
	iscan=0
	idet=0
	iframe=0
	subset=0
	FOR i=0L, nlines-1 DO BEGIN
		IF scan[i] EQ iscan+1 AND det[i] EQ idet+1 THEN BEGIN
			dn_frame[ham[i],idet,subset,iscan,iframe]=dns[i]
			iframe=iframe+1
			IF iframe EQ max_frame THEN BEGIN
				idet=idet+1
				iframe=0
				IF idet EQ detectors_TEB[b] THEN BEGIN
					idet=0
					iscan=iscan+1
				ENDIF
			ENDIF
		ENDIF
		IF a4 EQ 'I' THEN BEGIN
			IF subset EQ 0 THEN BEGIN
				subset=1
				goto,skip_over_1
			ENDIF
			IF subset EQ 1 THEN subset=0
			skip_over_1:
		ENDIF
	ENDFOR

;*****************************************************************************
;  Average over frames and calculate the standard deviation.
;*****************************************************************************

	FOR iham=0, nham-1 DO BEGIN
		FOR idet=0, detectors_TEB[b]-1 DO BEGIN
			FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
				FOR iscan=0, max_scan-1 DO BEGIN
					dn_col[a5,b,iham,idet,isubset,iscan,1]= $
						sig_mean(dn_frame[iham,idet,isubset,iscan,0:max_frame-1],3,1)
					std_col[a5,b,iham,idet,isubset,iscan,1]= $
						sig_mean(dn_frame[iham,idet,isubset,iscan,0:max_frame-1],3,2)

					IF dn_col[a5,b,iham,idet,isubset,iscan,1] EQ -999 THEN $
						dn_col[a5,b,iham,idet,isubset,iscan,1]=0
					IF std_col[a5,b,iham,idet,isubset,iscan,1] EQ -999 THEN $
						std_col[a5,b,iham,idet,isubset,iscan,1]=0
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR

;*****************************************************************************
;  Relabel the higher subsample as 0 and the lower as 1.
;*****************************************************************************

	temp1=dblarr(nham,maxdet,maxsubsets,max_scan)
	IF a4 EQ 'I' THEN BEGIN
		FOR iham=0, nham-1 DO BEGIN
			FOR idet=0, detectors_TEB[b]-1 DO BEGIN
				FOR iscan=0, max_scan-1 DO BEGIN
					IF dn_col[a5,b,iham,idet,0,iscan,1] GT dn_col[a5,b,iham,idet,1,iscan,1] THEN BEGIN
						temp1[iham,idet,0,iscan]=dn_col[a5,b,iham,idet,0,iscan,1]
						temp1[iham,idet,1,iscan]=dn_col[a5,b,iham,idet,1,iscan,1]
					ENDIF ELSE BEGIN
						temp1[iham,idet,0,iscan]=dn_col[a5,b,iham,idet,1,iscan,1]
						temp1[iham,idet,1,iscan]=dn_col[a5,b,iham,idet,0,iscan,1]
					ENDELSE
				ENDFOR
			ENDFOR
		ENDFOR
		dn_col[a5,b,*,*,0,*,1]=temp1[*,*,0,*]
		dn_col[a5,b,*,*,1,*,1]=temp1[*,*,1,*]
	ENDIF
ENDFOR

;*****************************************************************************
;  Read in the dump files for EV.
;*****************************************************************************

q=0
a5=0
FOR ifile=0, nfile-1 DO BEGIN

	nlines=File_Lines(list_files[ifile])
	IF nlines EQ 0 THEN continue

	a2=strsplit(list_files[ifile],'/',/Extract)
	a3=N_ELEMENTS(a2)
	a4=strmid(a2[a3-1],strpos(a2[a3-1],'_',/Reverse_Search)+1,1)		; Determine I or M band.
	a8=strmid(a2[a3-1],strpos(a2[a3-1],'/',/Reverse_Search)+10,2)		; Determine EV or SV.

	; Keep only the EV.
	IF a8 EQ 'SV' OR a8 EQ 'OB' THEN continue

	; Determine the relative collect #.
	q=q+1
	IF q EQ 9 THEN BEGIN
		a5=a5+1
		q=1
	ENDIF
	print,list_files[ifile]

	read_dmpd2,list_files[ifile],header_size,uaid,collect,scan,ham,apid,band,dnb_s,det,dsect,frame,offset,gain_type,dns

	; Translate the band name to a number.
	IF band[0] EQ 'I4' THEN b=0
	IF band[0] EQ 'I5' THEN b=1
	IF band[0] EQ 'M12' THEN b=2
	IF band[0] EQ 'M13' THEN b=3
	IF band[0] EQ 'M14' THEN b=4
	IF band[0] EQ 'M15' THEN b=5
	IF band[0] EQ 'M16A' THEN b=6
	IF band[0] EQ 'M16B' THEN b=7

	IF band_type_TEB[b] EQ 1 THEN max_frame=400		; Total # frames (I band).
	IF band_type_TEB[b] EQ 0 THEN max_frame=200

;*****************************************************************************
;  Write DN values to an array for EV.
;*****************************************************************************

	dn_frame=dblarr(nham,maxdet,maxsubsets,max_scan,max_frame)
	iscan=0
	idet=0
	iframe=0
	subset=0
	FOR i=0L, nlines-1 DO BEGIN
		IF scan[i] EQ iscan+1 AND det[i] EQ idet+1 THEN BEGIN
			dn_frame[ham[i],idet,subset,iscan,iframe]=dns[i]
			iframe=iframe+1
			IF iframe EQ max_frame THEN BEGIN
				idet=idet+1
				iframe=0
				IF idet EQ detectors_TEB[b] THEN BEGIN
					idet=0
					iscan=iscan+1
				ENDIF
			ENDIF
		ENDIF
		IF a4 EQ 'I' THEN BEGIN
			IF subset EQ 0 THEN BEGIN
				subset=1
				goto,skip_over_2
			ENDIF
			IF subset EQ 1 THEN subset=0
			skip_over_2:
		ENDIF
	ENDFOR

;*****************************************************************************
;  Relabel the higher subsample as 0 and the lower as 1.
;*****************************************************************************

	temp2=dblarr(nham,maxdet,maxsubsets,max_scan,max_frame)
	FOR iham=0, nham-1 DO BEGIN
		FOR idet=0, detectors_TEB[b]-1 DO BEGIN
			FOR iscan=0, max_scan-1 DO BEGIN
				IF sig_mean(dn_frame[iham,idet,0,iscan,0:max_frame-1],3,1) GT $
					sig_mean(dn_frame[iham,idet,1,iscan,0:max_frame-1],3,1) THEN BEGIN
					temp2[iham,idet,0,iscan,*]=dn_frame[iham,idet,0,iscan,*]
					temp2[iham,idet,1,iscan,*]=dn_frame[iham,idet,1,iscan,*]
				ENDIF ELSE BEGIN
					temp2[iham,idet,0,iscan,*]=dn_frame[iham,idet,1,iscan,*]
					temp2[iham,idet,1,iscan,*]=dn_frame[iham,idet,0,iscan,*]
				ENDELSE
			ENDFOR
		ENDFOR
	ENDFOR
	dn_frame[*,*,0,*,*]=temp2[*,*,0,*,*]
	dn_frame[*,*,1,*,*]=temp2[*,*,1,*,*]

;*****************************************************************************
;  Subtract the SV average over frames from each frame of EV.
;*****************************************************************************

	dn_col_1=dblarr(nham,maxdet,maxsubsets,max_scan,max_frame,2)		; (0=EV, 1=OBC)
	FOR iham=0, nham-1 DO BEGIN
		FOR idet=0, detectors_TEB[b]-1 DO BEGIN
			FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
				FOR iscan=0, max_scan-1 DO BEGIN
					FOR iframe=0, max_frame-1 DO BEGIN
						IF dn_frame[iham,idet,isubset,iscan,iframe] NE 0 THEN $
							dn_col_1[iham,idet,isubset,iscan,iframe,0]= $
							dn_frame[iham,idet,isubset,iscan,iframe] $
							-dn_col[a5,b,iham,idet,isubset,iscan,1]
						IF dn_frame[iham,idet,isubset,iscan,iframe] EQ 0 THEN $
							dn_col_1[iham,idet,isubset,iscan,iframe,0]=0
					ENDFOR
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR

;*****************************************************************************
;  Average over frames with standard deviation.
;*****************************************************************************

	FOR iham=0, nham-1 DO BEGIN
		FOR idet=0, detectors_TEB[b]-1 DO BEGIN
			FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
				FOR iscan=0, max_scan-1 DO BEGIN
					dn_col[a5,b,iham,idet,isubset,iscan,0]= $
						sig_mean(dn_col_1[iham,idet,isubset,iscan,0:max_frame-1,0],3,1)
					std_col[a5,b,iham,idet,isubset,iscan,0]= $
						sig_mean(dn_col_1[iham,idet,isubset,iscan,0:max_frame-1,0],3,2)
					IF dn_col[a5,b,iham,idet,isubset,iscan,0] EQ -999 THEN $
						dn_col[a5,b,iham,idet,isubset,iscan,0]=0
					IF std_col[a5,b,iham,idet,isubset,iscan,0] EQ -999 THEN $
						std_col[a5,b,iham,idet,isubset,iscan,0]=0
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR

;*****************************************************************************
;  Calculate drifting of dn over frames using a straight line fit.
;*****************************************************************************

	f=findgen(max_frame)
	y=dblarr(max_frame)
	FOR iham=0, nham-1 DO BEGIN
		FOR idet=0, detectors_TEB[b]-1 DO BEGIN
			FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
				FOR iscan=0, max_scan-1 DO BEGIN
					y[*]=dn_frame[iham,idet,isubset,iscan,*]
					result1=poly_fit(f,y,1)

					slope1[a5,b,iham,idet,isubset,iscan,*]=result1[*]
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR

ENDFOR

;*****************************************************************************
;  Read in the dump files for OBC.
;*****************************************************************************

q=0
a5=0
FOR ifile=0, nfile-1 DO BEGIN

	nlines=File_Lines(list_files[ifile])
	IF nlines EQ 0 THEN continue

	a2=strsplit(list_files[ifile],'/',/Extract)
	a3=N_ELEMENTS(a2)
	a4=strmid(a2[a3-1],strpos(a2[a3-1],'_',/Reverse_Search)+1,1)		; Determine I or M band.
	a8=strmid(a2[a3-1],strpos(a2[a3-1],'/',/Reverse_Search)+10,2)		; Determine EV or SV.

	; Output file name.
	outfile='/home/jmcintire/VIIRS_Data/RC_5_EDU/'+a2[a3-4]+'/'+a2[a3-3]+'/'+a2[a3-2]+'/' $
		+a2[a3-4]+'.'+a2[a3-3]+'.'+a2[a3-2]+'.dn.sav'

	; Keep only the OBC.
	IF a8 EQ 'SV' OR a8 EQ 'EV' THEN continue

	; Determine the relative collect #.
	q=q+1
	IF q EQ 9 THEN BEGIN
		a5=a5+1
		q=1
	ENDIF
	print,list_files[ifile]

	read_dmpd2,list_files[ifile],0,uaid,collect,scan,ham,apid,band,dnb_s,det,dsect,frame,offset,gain_type,dns

	; Translate the band name to a number.
	IF band[0] EQ 'I4' THEN b=0
	IF band[0] EQ 'I5' THEN b=1
	IF band[0] EQ 'M12' THEN b=2
	IF band[0] EQ 'M13' THEN b=3
	IF band[0] EQ 'M14' THEN b=4
	IF band[0] EQ 'M15' THEN b=5
	IF band[0] EQ 'M16A' THEN b=6
	IF band[0] EQ 'M16B' THEN b=7

	IF band_type_TEB[b] EQ 1 THEN max_frame=96		; Total # frames (I band).
	IF band_type_TEB[b] EQ 0 THEN max_frame=48		; Total # frames (M band).

;*****************************************************************************
;  Write DN values to an array for OBC.
;*****************************************************************************

	dn_frame=dblarr(nham,maxdet,maxsubsets,max_scan,max_frame)
	iscan=0
	idet=0
	iframe=0
	subset=0
	FOR i=0L, nlines-1 DO BEGIN
		IF scan[i] EQ iscan+1 AND det[i] EQ idet+1 THEN BEGIN
			dn_frame[ham[i],idet,subset,iscan,iframe]=dns[i]
			iframe=iframe+1
			IF iframe EQ max_frame THEN BEGIN
				idet=idet+1
				iframe=0
				IF idet EQ detectors_TEB[b] THEN BEGIN
					idet=0
					iscan=iscan+1
				ENDIF
			ENDIF
		ENDIF
		IF a4 EQ 'I' THEN BEGIN
			IF subset EQ 0 THEN BEGIN
				subset=1
				goto,skip_over_3
			ENDIF
			IF subset EQ 1 THEN subset=0
			skip_over_3:
		ENDIF
	ENDFOR

;*****************************************************************************
;  Relabel the higher subsample as 0 and the lower as 1.
;*****************************************************************************

	temp2=dblarr(nham,maxdet,maxsubsets,max_scan,max_frame)
	FOR iham=0, nham-1 DO BEGIN
		FOR idet=0, detectors_TEB[b]-1 DO BEGIN
			FOR iscan=0, max_scan-1 DO BEGIN
				IF mean(dn_frame[iham,idet,0,iscan,0:2]) GT $
					mean(dn_frame[iham,idet,1,iscan,3:5]) THEN BEGIN
					temp2[iham,idet,0,iscan,*]=dn_frame[iham,idet,0,iscan,*]
					temp2[iham,idet,1,iscan,*]=dn_frame[iham,idet,1,iscan,*]
				ENDIF ELSE BEGIN
					temp2[iham,idet,0,iscan,*]=dn_frame[iham,idet,1,iscan,*]
					temp2[iham,idet,1,iscan,*]=dn_frame[iham,idet,0,iscan,*]
				ENDELSE
			ENDFOR
		ENDFOR
	ENDFOR
	dn_frame[*,*,0,*,*]=temp2[*,*,0,*,*]
	dn_frame[*,*,1,*,*]=temp2[*,*,1,*,*]

;*****************************************************************************
;  Subtract the SV average over frames from each frame of OBC.
;*****************************************************************************

	dn_col_1=dblarr(nham,maxdet,maxsubsets,max_scan,max_frame,2)		; (0=EV, 1=OBC)
	FOR iham=0, nham-1 DO BEGIN
		FOR idet=0, detectors_TEB[b]-1 DO BEGIN
			FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
				FOR iscan=0, max_scan-1 DO BEGIN
					FOR iframe=0, max_frame-1 DO BEGIN
						IF dn_frame[iham,idet,isubset,iscan,iframe] NE 0 THEN $
							dn_col_1[iham,idet,isubset,iscan,iframe,1]= $
							dn_frame[iham,idet,isubset,iscan,iframe] $
							-dn_col[a5,b,iham,idet,isubset,iscan,1]
						IF dn_frame[iham,idet,isubset,iscan,iframe] EQ 0 THEN $
							dn_col_1[iham,idet,isubset,iscan,iframe,1]=0
					ENDFOR
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR

;*****************************************************************************
;  Average over frames with standard deviation.
;*****************************************************************************

	FOR iham=0, nham-1 DO BEGIN
		FOR idet=0, detectors_TEB[b]-1 DO BEGIN
			FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
				FOR iscan=0, max_scan-1 DO BEGIN
					dn_col[a5,b,iham,idet,isubset,iscan,2]= $
						sig_mean(dn_col_1[iham,idet,isubset,iscan,0:max_frame-1,1],3,1)
					std_col[a5,b,iham,idet,isubset,iscan,2]= $
						sig_mean(dn_col_1[iham,idet,isubset,iscan,0:max_frame-1,1],3,2)

					IF dn_col[a5,b,iham,idet,isubset,iscan,2] EQ -999 THEN $
						dn_col[a5,b,iham,idet,isubset,iscan,2]=0
					IF std_col[a5,b,iham,idet,isubset,iscan,2] EQ -999 THEN $
						std_col[a5,b,iham,idet,isubset,iscan,2]=0
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR

;*****************************************************************************
;  Calculate drifting of dn over frames using a straight line fit.
;*****************************************************************************

	f=findgen(max_frame)
	y=dblarr(max_frame)
	FOR iham=0, nham-1 DO BEGIN
		FOR idet=0, detectors_TEB[b]-1 DO BEGIN
			FOR isubset=0, subsets_TEB[b]-1 DO BEGIN
				FOR iscan=0, max_scan-1 DO BEGIN
					y[*]=dn_frame[iham,idet,isubset,iscan,*]
					result1=poly_fit(f,y,1)

					slope_obc1[a5,b,iham,idet,isubset,iscan,*]=result1[*]
				ENDFOR
			ENDFOR
		ENDFOR
	ENDFOR

ENDFOR

save,/compress,slope1,slope_obc1,dn_col,std_col,filename=outfile

skipout:

END


